@@ -1,6 +1,6 @@
 """
 SocksiPy - Python SOCKS module.
-Version 1.6.1
+Version 1.6.4
 
 Copyright 2006 Dan-Haim. All rights reserved.
 
@@ -52,7 +52,7 @@ Modifications made by Anorov (https://github.com/Anorov)
 -Various small bug fixes
 """
 
-__version__ = "1.6.1"
+__version__ = "1.6.4"
 
 import socket
 import struct
@@ -60,19 +60,16 @@ from errno import EOPNOTSUPP, EINVAL, EAGAIN
 from io import BytesIO
 from os import SEEK_CUR
 import os
+import sys
 from collections import Callable
 from base64 import b64encode
 
 
-if os.name == 'nt':
+if os.name == "nt" and sys.version_info < (3, 0):
     try:
         import win_inet_pton
-        import socket
     except ImportError:
-        raise ImportError('To run PySocks under windows you need to install win_inet_pton') 
-else:
-    import socket
-
+        raise ImportError("To run PySocks on Windows you must install win_inet_pton")
 
 PROXY_TYPE_SOCKS4 = SOCKS4 = 1
 PROXY_TYPE_SOCKS5 = SOCKS5 = 2
@@ -270,6 +267,8 @@ class socksocket(_BaseSocket):
         self.proxy_sockname = None
         self.proxy_peername = None
 
+        self._timeout = None
+
     def _readall(self, file, count):
         """
         Receive EXACTLY the number of bytes requested from the file object.
@@ -283,6 +282,24 @@ class socksocket(_BaseSocket):
             data += d
         return data
 
+    def settimeout(self, timeout):
+        self._timeout = timeout
+        try:
+            # test if we're connected, if so apply timeout
+            peer = self.get_proxy_peername()
+            _BaseSocket.settimeout(self, self._timeout)
+        except socket.error:
+            pass
+
+    def gettimeout(self):
+        return self._timeout
+
+    def setblocking(self, v):
+        if v:
+            self.settimeout(None)
+        else:
+            self.settimeout(0.0)
+
     def set_proxy(self, proxy_type=None, addr=None, port=None, rdns=True, username=None, password=None):
         """set_proxy(proxy_type, addr[, port[, rdns[, username[, password]]]])
         Sets the proxy to be used.
@@ -341,6 +358,7 @@ class socksocket(_BaseSocket):
         host, _ = proxy
         _, port = relay
         _BaseSocket.connect(self, (host, port))
+        _BaseSocket.settimeout(self, self._timeout)
         self.proxy_sockname = ("0.0.0.0", 0)  # Unknown
 
     def sendto(self, bytes, *args, **kwargs):
@@ -374,8 +392,8 @@ class socksocket(_BaseSocket):
         if not self._proxyconn:
             self.bind(("", 0))
 
-        buf = BytesIO(_BaseSocket.recv(self, bufsize, flags))
-        buf.seek(+2, SEEK_CUR)
+        buf = BytesIO(_BaseSocket.recv(self, bufsize + 1024, flags))
+        buf.seek(2, SEEK_CUR)
         frag = buf.read(1)
         if ord(frag):
             raise NotImplementedError("Received UDP packet fragment")
@@ -386,7 +404,7 @@ class socksocket(_BaseSocket):
             if fromhost != peerhost or peerport not in (0, fromport):
                 raise socket.error(EAGAIN, "Packet filtered")
 
-        return (buf.read(), (fromhost, fromport))
+        return (buf.read(bufsize), (fromhost, fromport))
 
     def recv(self, *pos, **kw):
         bytes, _ = self.recvfrom(*pos, **kw)
@@ -507,6 +525,8 @@ class socksocket(_BaseSocket):
 
             # Get the bound address/port
             bnd = self._read_SOCKS5_address(reader)
+
+            _BaseSocket.settimeout(self, self._timeout)
             return (resolved, bnd)
         finally:
             reader.close()
@@ -731,6 +751,7 @@ class socksocket(_BaseSocket):
         if proxy_type is None:
             # Treat like regular socket object
             self.proxy_peername = dest_pair
+            _BaseSocket.settimeout(self, self._timeout)
             _BaseSocket.connect(self, (dest_addr, dest_port))
             return
 
@@ -757,6 +778,7 @@ class socksocket(_BaseSocket):
                 # Calls negotiate_{SOCKS4, SOCKS5, HTTP}
                 negotiate = self._proxy_negotiators[proxy_type]
                 negotiate(self, dest_addr, dest_port)
+                _BaseSocket.settimeout(self, self._timeout)
             except socket.error as error:
                 # Wrap socket errors
                 self.close()
